# -*- coding: utf-8 -*-
"""
微信公众号开发相关功能模块
基于django1.8、wechat_sdk0.6.3，暂时提供了一些好用的修饰器
"""
from wechat_decorators import require_wechat_authorization, require_wechat_config

__all__ = [
    'require_wechat_authorization',
    'require_wechat_config',
]


# -*- coding: utf-8 -*-
import os
import six
import string
import random
import time
import hashlib

from django.conf import settings

from wechat_sdk import WechatConf
from django.core.cache import caches
cache = caches['default']



# 获取view中get方法的参数公共方法
def get_get_args(request, *args):
    get_args = []
    for item in args:
        get_args.append(request.GET.get(item, None))

    return get_args


def get_nonce_str(length):
    return ''.join(random.choice(string.ascii_letters + string.digits) for _ in xrange(length))


def get_timestamp():
        return int(time.time())


def sign_js_wechat_config(config):
    ret = []
    for key in sorted(config.keys()):
        if (key != 'signature') and (key != '') and (config[key] is not None):
            ret.append('%s=%s' % (key.lower(), config[key]))
    sign_string = '&'.join(ret)
    config['signature'] = hashlib.sha1(sign_string.encode('utf-8')).hexdigest()



####################################
# 微信订阅号开发 #
# 获取配置文件＆＆管理各种token #
####################################
def get_wechat_conf(appid=None, appsecret=None, token=None, encoding_aes_key=None, encrypt_mode='normal', **kwargs):
    # 从缓存中获取各种token
    access_token = cache.get('wechat_access_token_%s' % appid)
    access_token_expires_at = cache.get('wechat_access_token_expires_at_%s' % appid)
    jsapi_ticket = cache.get('wechat_jsapi_ticket_%s' % appid)
    jsapi_ticket_expires_at = cache.get('wechat_jsapi_ticket_expires_at_%s' % appid)

    # 配置
    conf = WechatConf( 
        token=token, 
        appid=appid, 
        appsecret=appsecret, 
        encrypt_mode=encrypt_mode,  # 可选项：normal/compatible/safe，分别对应于 明文/兼容/安全 模式
        encoding_aes_key=encoding_aes_key, # 如果传入此值则必须保证同时传入 token, appid
        access_token=access_token,
        access_token_expires_at=access_token_expires_at,
        jsapi_ticket=jsapi_ticket,
        jsapi_ticket_expires_at=jsapi_ticket_expires_at,
    )

    # 获取最新的token，设置缓存（上一步配置的时候，如果没有过期，还是得到原来的token）
    access_token_newest = conf.get_access_token()
    cache.set('wechat_access_token_%s' % appid, access_token_newest.get('access_token'))
    cache.set('wechat_access_token_expires_at_%s' % appid, access_token_newest.get('access_token_expires_at'))
    jsapi_ticket_newest = conf.get_jsapi_ticket()
    cache.set('wechat_jsapi_ticket_%s' % appid, jsapi_ticket_newest.get('jsapi_ticket'))
    cache.set('wechat_jsapi_ticket_expires_at_%s' % appid, jsapi_ticket_newest.get('jsapi_ticket_expires_at'))

    return conf



def get_wechat_config(name, default=None):
    """
    Get configuration variable from environment variable
    or django setting.py
    """
    config = os.environ.get(name, getattr(settings, name, default))
    if config is not None:
        if isinstance(config, six.string_types):
            return config.strip()
        else:
            return config
    else:
        return None


# -*- coding:utf-8 -*-
"""
对于第三方包wechat_sdk的功能扩展
"""
import json
import requests
import six
from wechat_sdk import WechatBasic
from wechat_sdk.lib.request import WechatRequest




class WechatRequestExtension(WechatRequest):
    """ 
    修改WechatRequest对传过去的参数access_token的处理（原始默认是使用账号的access_token，而微信网页相关接口需要直接使用网页的access_token）
    """
    def __init__(self, conf=None):
        """
        :param conf: WechatConf 配置类实例
        """
        self.__conf = conf

    def request(self, method, url, access_token=None, **kwargs):
        """
        向微信服务器发送请求
        :param method: 请求方法
        :param url: 请求地址
        :param access_token: access token 值, 如果初始化时传入 conf 会自动获取, 如果没有传入则请提供此值
        :param kwargs: 附加数据
        :return: 微信服务器响应的 JSON 数据
        """
        access_token = self.__conf.access_token if self.__conf is not None else access_token
        if "params" not in kwargs:
            kwargs["params"] = {
                "access_token": access_token
            }
        elif 'access_token' not in  kwargs["params"]:
            kwargs["params"]["access_token"] = access_token

        if isinstance(kwargs.get("data", ""), dict):
            body = json.dumps(kwargs["data"], ensure_ascii=False)
            if isinstance(body, six.text_type):
                body = body.encode('utf8')
            kwargs["data"] = body

        r = requests.request(method=method, url=url, **kwargs )
        r.raise_for_status()

        ######## 此处注意，微信js相关接口返回的response的encoding有误，导致r.json()编码错误
        # 使用r.encoding = None解决
        try:
            r.encoding = None
            response_json = r.json()
        except ValueError:  # 非 JSON 数据
            return r

        headimgurl = response_json.get('headimgurl')
        if headimgurl:
            response_json['headimgurl'] = headimgurl.replace('\\', '')
        self._check_official_error(response_json)
        return response_json





class WechatBasicExtension(WechatBasic):
    """
    新增一些sdk没有的功能
    """

    def get_js_authorization_redirect_url(self, redirect_uri, scope='snsapi_userinfo', state=''):
        """
        微信网页授权流程第一步：用户同意授权，获取code
        返回：用户授权回调url
        """
        if not redirect_uri:
            raise ValueError('Parameter redirect_uri is not valid')
        return 'https://open.weixin.qq.com/connect/oauth2/authorize?appid=%s&redirect_uri=%s&response_type=code&scope=%s&state=%s#wechat_redirect' % (self.conf.appid, redirect_uri, scope, state, )

    def get_js_authorization_access_token(self, code):
        """
        微信网页授权流程第二步：通过code换取网页授权access_token
        :param code 填写第一步获取的code参数
        :return: access_token    网页授权接口调用凭证,注意：此access_token与基础支持的access_token不同
        :return: expires_in  access_token接口调用凭证超时时间，单位（秒）
        :return: refresh_token   用户刷新access_token
        :return: openid  用户唯一标识，请注意，在未关注公众号时，用户访问公众号的网页，也会产生一个用户和公众号唯一的OpenID
        :return: scope   用户授权的作用域，使用逗号（,）分隔
        """
        return self.request.get(
            url='https://api.weixin.qq.com/sns/oauth2/access_token',
            params={
                'appid': self.conf.appid,
                'secret': self.conf.appsecret,
                'code': code,
                'grant_type': 'authorization_code',
            }
        )

    def get_js_authorization_refresh_token(self, refresh_token):
        """
        微信网页授权流程第三步：刷新access_token（如果需要）
        :param refresh_token 填写通过access_token获取到的refresh_token参数  
        :return: access_token    网页授权接口调用凭证,注意：此access_token与基础支持的access_token不同
        :return: expires_in  access_token接口调用凭证超时时间，单位（秒）
        :return: refresh_token   用户刷新access_token
        :return: openid  用户唯一标识，请注意，在未关注公众号时，用户访问公众号的网页，也会产生一个用户和公众号唯一的OpenID
        :return: scope   用户授权的作用域，使用逗号（,）分隔
        """

        return self.request.get(
            url='https://api.weixin.qq.com/sns/oauth2/refresh_token',
            params={
                'appid': self.conf.appid,
                'grant_type': 'refresh_token',
                'refresh_token': refresh_token,
            }
        )

    def get_js_authorization_user_info(self, access_token, openid, lang='zh_CN'):
        """
        微信网页授权流程第四步：拉取用户信息(需scope为 snsapi_userinfo)
        :param access_token 网页授权接口调用凭证,注意：此access_token与基础支持的access_token不同
        :param openid   用户的唯一标识
        :return: 返回的 JSON 数据包
        """
        # 注意此处要使用到jsSDK的access_token,则使用扩展的WechatRequestExtension
        request = WechatRequestExtension(conf=self.conf)
        return request.get(
            url='https://api.weixin.qq.com/sns/userinfo',
            params={
                'access_token': access_token,
                'openid': openid,
                'lang': lang,
            }
        )






# -*- coding: utf-8 -*-
"""
微信公众号实用修饰器
"""
from django.http import HttpResponseRedirect

from wechat_api import WeChatApi
from wechat_conf import WECHAT_APPID, WECHAT_APPSECRET, WECHAT_JS_DOMAIN, WECHAT_TOKEN, WECHAT_ENCODING_AES_KEY, WECHAT_MODE
from wechat_utils import get_get_args

# a decorator, used on any view that needs user_info
# user_info will be passed as a key word argument to the view
# NOTE:
# user_info may be None if the given params are wrong,
# since there has not a global solution to indicate what to do at this situation,
# the judgement is left to the view to handle.
# 微信网页授权,　获取用户信息
def require_wechat_authorization(appid=None, appsecret=None, js_domain=None, token=None, encoding_aes_key=None, encrypt_mode=None):
    def decorator(func):
        def wrapper(request, *args, **kwargs):
            wechat_api = WeChatApi(
                appid = appid or WECHAT_APPID,
                appsecret =appsecret or WECHAT_APPSECRET,
                js_domain =js_domain or WECHAT_JS_DOMAIN,
                token =token or WECHAT_TOKEN,
                encoding_aes_key = encoding_aes_key or WECHAT_ENCODING_AES_KEY,
                encrypt_mode = encrypt_mode or WECHAT_MODE
            )
            identify = request.get_signed_cookie('identify', default=None, salt='js_authorization')
            cached_userinfo = wechat_api.get_js_user_info_by_openid(identify) if identify else None
            user_info = None
            if not cached_userinfo:
                code, state = get_get_args(request, 'code', 'state')
                if not state or state != 'authorized':
                    redirect_uri = '%s://%s%s' % (request.scheme, wechat_api.js_domain, request.path)
                    authorization_redirect_url = wechat_api.wechat_basic.get_js_authorization_redirect_url(redirect_uri, scope='snsapi_userinfo', state='authorized')
                    return HttpResponseRedirect(authorization_redirect_url)
                user_info = wechat_api.get_js_user_info_by_code(code)
            else :
                user_info = cached_userinfo
            kwargs.update({'wechat_user_info': user_info})
            response = func(request, *args, **kwargs)
            response.set_signed_cookie('identify', value=user_info['openid'], salt='js_authorization')
            return response
        return wrapper
    return decorator


# 获取wechat　config,作为前台wx.config使用
def require_wechat_config(appid=None, appsecret=None, js_domain=None, token=None, encoding_aes_key=None, encrypt_mode=None):
    def decorator(func):
        def wrapper(request, *args, **kwargs):
            wechat_api = WeChatApi(
                appid = appid or WECHAT_APPID,
                appsecret =appsecret or WECHAT_APPSECRET,
                js_domain =js_domain or WECHAT_JS_DOMAIN,
                token =token or WECHAT_TOKEN,
                encoding_aes_key = encoding_aes_key or WECHAT_ENCODING_AES_KEY,
                encrypt_mode = encrypt_mode or WECHAT_MODE
            )
            wechat_config = wechat_api.get_js_wechat_config(request.get_full_path())
            kwargs.update({'wechat_config':wechat_config})
            response = func(request, *args, **kwargs)
            return response
        return wrapper
    return decorator


from django.contrib import admin
from vote.models import *

class VoteActivityAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', )

class VoteImageAdmin(admin.ModelAdmin):
    list_display = ('image', )

admin.site.register(VoteImage, VoteImageAdmin)
admin.site.register(VoteActivity, VoteActivityAdmin)


# -*- coding: utf-8 -*-
from wechat_utils import get_wechat_config

WECHAT_APPID = get_wechat_config('WECHAT_APPID')
WECHAT_APPSECRET = get_wechat_config('WECHAT_APPSECRET')
WECHAT_JS_DOMAIN = get_wechat_config('WECHAT_JS_DOMAIN')
WECHAT_TOKEN = get_wechat_config('WECHAT_TOKEN')
WECHAT_ENCODING_AES_KEY = get_wechat_config('WECHAT_ENCODING_AES_KEY')
WECHAT_MODE = get_wechat_config('WECHAT_MODE', 'normal')


# -*- coding: utf-8 -*-
import json
from decimal import Decimal

class FieldVolidatorMixin(object):
    """
    参数验证
    """
    validate_key = [
        'exact_length',
        'max_length',
        'min_length',
        'exact_value',
        'max_value',
        'min_value',
    ]

    def validate_field(self, value, validators):
        for validator in validators:
            if hasattr(validator, '__call__'):
                if not self.func_validate(validator, value):
                    return False
            else:
                for vali_key,vali_value in validator.items():
                    if not self.general_validate(vali_key, vali_value, value):
                        return False
        return True

    def func_validate(self, func, value):
        return func(value)

    def general_validate(self, vali_key, vali_value, value):
        if vali_key not in self.validate_key:
            raise Exception('validate key %s is not valid' % vali_key)
        return getattr(self, '%s_validate' % vali_key)(vali_value, value)

    def exact_length_validate(self, vali_value, value):
        return int(vali_value) == len(value)

    def max_length_validate(self, vali_value, value):
        return int(vali_value) >= len(value)

    def min_length_validate(self, vali_value, value):
            return int(vali_value) <= len(value) 

    def exact_value_validate(self, vali_value, value):
        return vali_value == value

    def max_value_validate(self, vali_value, value):
        return vali_value >= value

    def min_value_validate(self, vali_value, value):
        return vali_value <= value



class FieldFormatorMixin(object):
    """
    参数format
    """
    format_key = [
        'int',
        'unicode',
        'decimal',
    ]
    TWOPLACES = Decimal(10) ** -2

    def format_field(self, value, formator):
        if formator not in self.format_key:
            raise Exception('format key %s is not valid' % formator)
        return getattr(self, '%s_format' % formator)(value)

    def int_format(self, value):
        try:
            return int(value)
        except Exception, e:
            raise Exception('format error')

    def unicode_format(self, value):
        try:
            return unicode(value)
        except Exception, e:
            raise Exception('format error')

    def decimal_format(self, value):
        try:
            return Decimal(value).quantize(self.TWOPLACES)
        except Exception, e:
            raise Exception('format error')

        

class ReqParameter(FieldFormatorMixin, FieldVolidatorMixin):
    """
    参数处理
    """
    METHODS = ['POST', 'GET'] # 暂时支持的方法
    # 常用字段规则
    # FIELD_USERNAME = {'fieldname':'username', 'formator':'unicode', 'validators':[lambda x:x.isdigit(),{'exact_length':11}], 'option':False}
    # FIELD_PASSWORD = {'fieldname':'password', 'formator':'unicode', 'validators':[{'min_length':6, 'max_length':14}], 'option':False}
    # FIELD_PASSWORD_REPEAT = {'fieldname':'password_repeat', 'formator':'unicode', 'validators':[{'min_length':6, 'max_length':14}], 'option':False}
    # FIELD_PASSWORD_ORIGIN = {'fieldname':'origin_password', 'formator':'unicode', 'validators':[{'min_length':6, 'max_length':14}], 'option':False}
    # FIELD_SMS = {'fieldname':'sms', 'formator':'unicode', 'validators':[lambda x:x.isdigit(),{'exact_length':6}]}

    def __init__(self, request, fields=[]):
        self.request = request
        self.fields = fields

    
    def get_method_item_value(self, method, fieldname, args_info, defaultvalue=None, option=False):
        item_value = None
        if method == 'POST':
            if self.request.POST.get(fieldname, None) is None:
                item_value = args_info.get(fieldname, None)
            else:
                item_value = self.request.POST.get(fieldname, None)

        elif method == 'GET':
            item_value = self.request.GET.get(fieldname, None)

        # 判断如果是非必要字段，给默认值
        if item_value is None and option:
            item_value = defaultvalue

        return item_value

    def get_args(self, method):
        if method not in self.METHODS:
            raise Exception('ReqParameter have not allow this method:%s' % method)

        valid, filedvalues = True, []

        try:
            args_info = json.loads(self.request.body)
        except Exception, e:
            args_info = {}

        for item in self.fields:
            # 字段参数
            fieldname = item.get('fieldname')
            formator = item.get('formator')
            validators = item.get('validators', [])
            option = item.get('option', False)
            defaultvalue = item.get('defaultvalue')
            if fieldname is None:
                raise Exception('fields arg must have key : fieldname')

            item_value = self.get_method_item_value(method, fieldname, args_info, defaultvalue, option)

            # 验证参数是否为必要参数
            if not option and item_value is None:
                valid = False
                break

            if option and item_value is None:
                # 如果是非必要参数，且前端没有传此值，则无需做各种验证及转换
                pass
            else:
                # 进行format
                if formator is not None:
                    try:
                        item_value = self.format_field(item_value, formator)
                    except Exception, e:
                        valid = False
                        break
                # 进行数据验证
                if not self.validate_field(item_value, validators):
                    valid = False
                    break

            filedvalues.append(item_value)

        return valid, filedvalues


    def get_post_args(self):
        return self.get_args('POST')


    def get_get_args(self):
        return self.get_args('GET')


# -*- coding:utf-8 -*-
from vote.models import *
from tastypie.resources import ModelResource, ALL, ALL_WITH_RELATIONS
from tastypie import fields

__all__ = ['VoteProductResource',]



####################################
# 投票排行榜接口 #
# URL: /api/v1/vote/productbrief/
####################################
class VoteProductResource(ModelResource):

    activity_id = fields.IntegerField(attribute='activity_id_id', null=True)
    image = fields.CharField(attribute='image_first_url', default='')
    slug = fields.CharField(attribute='slug', default='')

    class Meta:
        queryset = VoteProduct.objects.filter(active=True)
        resource_name = 'productbrief'
        allowed_methods = ['get']
        excludes = ['create_uid', 'write_uid', 'write_date']
        # 单页最大可以返回的数量
        max_limit = 50
        # 前台get请求不带limit参数时默认单页返回的数量
        limit = 20
        ordering = ['vote_count','create_date']
        filtering = {
            'id': ALL,
            'activity_id':ALL,
        }

    def dehydrate(self, bundle):
        """
        默认id变为rid，前台格式需求（方便其数据库使用）
        """
        if bundle.data.has_key('id'):
            bundle.data['rid'] = bundle.data.pop('id')
        return bundle


# -*- coding: utf-8 -*-
import logging
from django.core.cache import caches

from wechat_sdk_extension import WechatBasicExtension
from wechat_utils import get_nonce_str, get_timestamp, sign_js_wechat_config, get_wechat_conf

cache = caches['default']
logger = logging.getLogger(__name__)




class WeChatApi(object):
    """
    微信公众号业务类
    """

    def __init__(self, appid, appsecret, js_domain=None, token=None, encoding_aes_key=None, encrypt_mode='normal'):
        self.js_domain = js_domain
        self.wechat_basic = WechatBasicExtension(get_wechat_conf(appid, appsecret, token, encoding_aes_key, encrypt_mode))


    def get_js_user_info_by_openid(self, openid):
        appId = self.wechat_basic.conf.appid
        cached_userinfo = cache.get('%s_%s_userinfo'%(appId, openid))
        if cached_userinfo:
            return cached_userinfo
        cached_openid_json = cache.get('%s_%s_openid'%(appId, openid))
        if not cached_openid_json:
            return None
        refresh_token = cached_openid_json['refresh_token']

        openid_json = self.wechat_basic.get_js_authorization_refresh_token(refresh_token)
        if openid_json.has_key('errcode'):
            logger.error('error when getting openid: %s, %s' %
                          (openid_json['errcode'], openid_json['errmsg']))
            return None

        # dont cache openid here, we don't know exact expire time for refresh_token here
        userinfo_json = self.wechat_basic.get_js_authorization_user_info(openid_json['access_token'], openid_json['openid'])
        if userinfo_json.has_key('errcode'):
            logger.error('error when getting userinfo: %s, %s' %
                          (userinfo_json['errcode'], userinfo_json['errmsg']))
            return None
        cache.set('%s_%s_userinfo'%(appId, openid_json['openid']), userinfo_json, timeout=int(openid_json['expires_in'])) # 7200s, 2h

        return userinfo_json


    # 网页授权流程
    # 1. 前端访问授权url获得code （见require_wechat_authorization）
    # 2. 使用code和appid与secret获取openid和access_token， appid与secret必须用服务号的，这里的access_token是特定的，不同于缓存里调用接口所用的token
    # 3. 使用2里获得的access_token和openid获取个人信息
    # 返回参数结构见
    # https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1421140842&token=&lang=zh_CN
    # 需要注意的是授权的回调url不支持参数
    def get_js_user_info_by_code(self, code):
        appId = self.wechat_basic.conf.appid

        openid_json = self.wechat_basic.get_js_authorization_access_token(code)
        if openid_json.has_key('errcode'):
            logger.error('error when getting openid: %s, %s' %
                          (openid_json['errcode'], openid_json['errmsg']))
            return None

        cache.set('%s_%s_openid'%(appId,openid_json['openid']), openid_json, timeout=2592000) #30days
        cached_userinfo = cache.get('%s_%s_userinfo'%(appId, openid_json['openid']))
        if cached_userinfo:
            return cached_userinfo

        userinfo_json = self.wechat_basic.get_js_authorization_user_info(openid_json['access_token'], openid_json['openid'])
        if userinfo_json.has_key('errcode'):
            logger.error('error when getting userinfo: %s, %s' %
                          (userinfo_json['errcode'], userinfo_json['errmsg']))
            return None
        cache.set('%s_%s_userinfo'%(appId, openid_json['openid']), userinfo_json, timeout=int(openid_json['expires_in'])) # 7200s, 2h

        return userinfo_json


    def get_js_wechat_config(self, full_path):
        ret = {
            'nonceStr': get_nonce_str(15),
            'jsapi_ticket': self.wechat_basic.conf.jsapi_ticket,
            'timestamp': get_timestamp(),
            'url': 'http://%s%s' % (self.js_domain, full_path, ),
        }
        sign_js_wechat_config(ret)
        ret['appId'] = self.wechat_basic.conf.appid
        return ret


# -*- coding: utf-8 -*-
from django.conf.urls import patterns, include, url

urlpatterns = [
    url(r'^activity/index/(?P<activity_id>[0-9]+)/$', 'vote.views.tmp_activity_index'),
    url(r'^product/index/(?P<product_id>[0-9]+)/$', 'vote.views.tmp_product_index'),
    url(r'^product/createform/(?P<activity_id>[0-9]+)/$', 'vote.views.tmp_product_createfrom'),
    url(r'^products/list/(?P<activity_id>[0-9]+)/$', 'vote.views.tmp_products_list'),
]


# -*- coding: utf-8 -*-
from tastypie.api import Api
from django.conf.urls import patterns, include, url

from vote.modelsResource import *


v1_api = Api(api_name='v1/vote')
v1_api.register(VoteProductResource())


urlpatterns = [
    url(r'^v1/vote/createproduct/$', 'vote.views.vote_create_product'),
    url(r'^v1/vote/submit/$', 'vote.views.vote_submit'),
]


# -*- coding:utf-8 -*-
import datetime
from django.utils import timezone
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.views.decorators.http import require_http_methods
from django.http import HttpResponse, JsonResponse, HttpResponseRedirect, Http404
from django.core.files import File
from django.db import transaction
from django.db.models import Q as modelsQ, F as modelsF
from wechat_lib import require_wechat_authorization, require_wechat_config
from common import ReqParameter
from vote.models import *


__all__ = [
    'vote_create_product', 'vote_submit', 'tmp_activity_index', 
    'tmp_product_createfrom', 'tmp_product_index', 'tmp_products_list',
]



####################################
# 投票内容上传接口 #
# URL: /api/v1/vote/createproduct/
####################################
@require_wechat_authorization()
@require_http_methods(["POST"])
def vote_create_product(request, *args, **kwargs):
    info = {'status':0, 'message':'success', 'product_id':''}
    product_data = {}
    # 参数公共处理
    parameters = ReqParameter(request, [
        {'fieldname':'activity_id', 'formator':'unicode', 'option':False},
        {'fieldname':'up_user_name', 'formator':'unicode', 'option':False},
        {'fieldname':'up_user_mobile', 'formator':'unicode', 'validators':[lambda x:x.isdigit(),{'exact_length':11}], 'option':False},
        {'fieldname':'content', 'formator':'unicode', 'option':True},
        ])
    p_valid, fieldvalues = parameters.get_post_args()
    if not p_valid:
        info['status'] = 1
        info['message'] = u'参数有误'
        return JsonResponse(info, safe=False)
    activity_id, up_user_name, up_user_mobile, content = fieldvalues
    f = request.FILES.get('image')
    if not f:
        return JsonResponse({'status':1, 'message':u'请上传图片', 'product_id':''}, safe=False)

    activity = get_object_or_404(VoteActivity, id=activity_id, active=True)
    # 加入上传规则
    wechat_user_info = kwargs.get('wechat_user_info')
    openid = wechat_user_info.get('openid')
    if activity.is_endtime_active:
        if datetime.datetime.now() >= timezone.make_naive(activity.end_time):
            info['status'] = 1
            info['message'] = u'sorry,活动已截止'
            return JsonResponse(info, safe=False)
    if activity.up_product_max != 0:
        if VoteProduct.objects.filter(activity_id_id=activity.id, up_user_slug=openid).count() >= activity.up_product_max:
            info['status'] = 1
            info['message'] = u'您已达到上传上限'
            return JsonResponse(info, safe=False)

    # 正式上传作品
    product_data['activity_id_id'] = activity.id
    product_data['activity_name'] = activity.name
    product_data['up_user_name'] = up_user_name
    product_data['up_user_mobile'] = up_user_mobile
    product_data['content'] = content
    try:
        with transaction.atomic(using='default', savepoint=True):
            vote_image = VoteImage.objects.create(image=File(f))
            wechat_fields = WeChatUser._meta.get_all_field_names()
            # wechat_user = WeChatUser(**dict(filter(lambda x:x[0] in wechat_fields, wechat_user_info.items())))
            # wechat_user.save()
            wechat_user, is_created = WeChatUser.objects.get_or_create(openid=openid, defaults=dict(filter(lambda x:x[0] in wechat_fields, wechat_user_info.items())))
            product_data['wechatuser'] = wechat_user
            product_data['up_user_slug'] = openid
            vote_product = VoteProduct.objects.create(**product_data)
            vote_product.images.add(vote_image)
    except Exception, e:
        info['status'] = 1
        info['message'] = u'上传有误'
        return JsonResponse(info, safe=False)

    return JsonResponse({'status':0, 'message':u'上传成功', 'product_id':vote_product.id}, safe=False)



####################################
# 投票接口 #
# URL: /api/v1/vote/submit/
####################################
@require_wechat_authorization()
@require_http_methods(["POST"])
def vote_submit(request, *args, **kwargs):
    info = {
        'status':0,
        'message':'success',
    }
    # 参数公共处理
    parameters = ReqParameter(request, [{'fieldname':'product_id', 'formator':'unicode', 'option':False}])
    p_valid, fieldvalues = parameters.get_post_args()
    if not p_valid:
        info['status'] = 1
        info['message'] = u'参数有误'
        return JsonResponse(info, safe=False)
    product_id, = fieldvalues

    product = get_object_or_404(VoteProduct, id=product_id, active=True)
    activity = product.activity_id
    # 加入投票规则
    wechat_user_info = kwargs.get('wechat_user_info')
    openid = wechat_user_info.get('openid')
    if activity.is_endtime_active:
        if datetime.datetime.now() >= timezone.make_naive(activity.end_time):
            info['status'] = 1
            info['message'] = u'sorry,活动已截止'
            return JsonResponse(info, safe=False)
    if activity.vote_activity_max != 0:
        if VoteItem.objects.filter(activity_id_id=activity.id, vote_user_slug=openid).count() >= activity.vote_activity_max:
            info['status'] = 1
            info['message'] = u'您已达到投票上限'
            return JsonResponse(info, safe=False)
    if activity.vote_product_max != 0:
        if VoteItem.objects.filter(product_id_id=product.id, vote_user_slug=openid).count() >= activity.vote_product_max:
            info['status'] = 1
            info['message'] = u'您已达到投票上限'
            return JsonResponse(info, safe=False)
    if not activity.vote_self_enabled:
        if product.up_user_slug == openid:
            info['status'] = 1
            info['message'] = u'您不可以投票给自己'
            return JsonResponse(info, safe=False)

    # 正式投票
    try:
        with transaction.atomic(using='default', savepoint=True):
            wechat_fields = WeChatUser._meta.get_all_field_names()
            # wechat_user = WeChatUser(**dict(filter(lambda x:x[0] in wechat_fields, wechat_user_info.items())))
            # wechat_user.save()
            wechat_user, is_created = WeChatUser.objects.get_or_create(openid=openid, defaults=dict(filter(lambda x:x[0] in wechat_fields, wechat_user_info.items())))
            VoteItem.objects.create(product_id_id=product.id, activity_id_id=activity.id, vote_user_slug=openid, wechatuser=wechat_user)
            product.vote_count = modelsF('vote_count') + 1
            product.save()
    except Exception, e:
        info['status'] = 1
        info['message'] = u'投票失败'
        return JsonResponse(info, safe=False)

    return JsonResponse(info, safe=False)





####################################
# 活动详情页 #
# URL: /vote/activity/index/1/
####################################
@require_wechat_authorization()
@require_wechat_config()
@require_http_methods(["GET"])
def tmp_activity_index(request, *args, **kwargs):
    activity_id = kwargs.get('activity_id')
    wechat_config = kwargs.get('wechat_config')
    activity = get_object_or_404(VoteActivity, id=activity_id, active=True)
    ctx = {'activity': activity}
    ctx.update(wechat_config)
    ctx.update({'linkurl':'%(host)s%(path)s' % {'host':request.get_host(),'path':request.path}})
    template_name = 'vote/vote_activity_index.html'
    return render_to_response(template_name, RequestContext(request, ctx))




####################################
# 投票上传页 #
# URL: /vote/product/createform/1/
####################################
@require_wechat_authorization()
@require_wechat_config()
@require_http_methods(["GET"])
def tmp_product_createfrom(request, *args, **kwargs):
    activity_id = kwargs.get('activity_id')
    wechat_config = kwargs.get('wechat_config')
    activity = get_object_or_404(VoteActivity, id=activity_id, active=True)
    ctx = {'activity': activity}
    ctx.update(wechat_config)
    ctx.update({'linkurl':'%(host)s%(path)s' % {'host':request.get_host(),'path':request.path}})
    template_name = 'vote/vote_product_createform.html'
    return render_to_response(template_name, RequestContext(request, ctx))



####################################
# 单个投票product详情页 #
# URL: /vote/product/index/1/
####################################
@require_wechat_authorization()
@require_wechat_config()
@require_http_methods(["GET"])
def tmp_product_index(request, *args, **kwargs):
    product_id = kwargs.get('product_id')
    product = get_object_or_404(VoteProduct, id=product_id, active=True)
    wechat_user_info = kwargs.get('wechat_user_info')
    wechat_config = kwargs.get('wechat_config')
    openid = wechat_user_info.get('openid')
    ctx = {'product': product, 'is_self': (openid==product.up_user_slug)}
    ctx.update(wechat_config)
    ctx.update({'linkurl':'%(host)s%(path)s' % {'host':request.get_host(),'path':request.path}})
    template_name = 'vote/vote_product_index.html'
    return render_to_response(template_name, RequestContext(request, ctx))



####################################
# 投票排行榜页 #
# URL: /vote/products/list/1/
####################################
@require_wechat_authorization()
@require_wechat_config()
@require_http_methods(["GET"])
def tmp_products_list(request, *args, **kwargs):
    activity_id = kwargs.get('activity_id')
    wechat_config = kwargs.get('wechat_config')
    activity = get_object_or_404(VoteActivity, id=activity_id, active=True)
    ctx = {'activity': activity}
    template_name = 'vote/vote_products_list.html'
    ctx.update(wechat_config)
    ctx.update({'linkurl':'%(host)s%(path)s' % {'host':request.get_host(),'path':request.path}})
    return render_to_response(template_name, RequestContext(request, ctx))




# -*- coding:utf-8 -*-
import codecs
import os
import sys

try:
    from setuptools import setup, find_packages
except:
    from distutils.core import setup

def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()



NAME = "django_wechatvote"

PACKAGES = find_packages()

REQUIRES = ['Django==1.8.2', 'requests', 'wechat-sdk', 'django-tastypie==0.12.2', 'qiniu', 'Pillow']

DESCRIPTION = "this is a package for django wechat vote"


KEYWORDS = "django vote wechat"

AUTHOR = "minkedong"

AUTHOR_EMAIL = "minkedong89@126.com"

URL = "https://github.com/minkedong/django_wechatvote"

VERSION = "0.0.4"

LICENSE = "Free"

LONG_DESCRIPTION = """
django_wechatvote

***微信投票活动***

文档地址：

http://blog.08050142.com/2016/08/12/django-wechatvote/

"""


setup(
    name = NAME,
    version = VERSION,
    description = DESCRIPTION,
    long_description = LONG_DESCRIPTION,
    classifiers = [],
    keywords = KEYWORDS,
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    url = URL,
    license = LICENSE,
    packages = PACKAGES,
    install_requires = REQUIRES,
    include_package_data=True,
    zip_safe=True,
)


# -*- coding:utf-8 -*-
import datetime
import uuid
from django.db import models
from django.utils import timezone
from qiniustorage.backends import QiniuMediaStorage

__all__ = ['VoteImage', 'VoteActivity', 'VoteProduct', 'VoteItem', 'WeChatUser']


class AbstractDefaultColumn(models.Model):
    create_uid = models.IntegerField(default=0)
    write_uid = models.IntegerField(default=0)
    create_date = models.DateTimeField(auto_now_add=True)  # 使用auto_now_add
    write_date = models.DateTimeField(auto_now=True)  # 使用auto_now

    class Meta:
        abstract = True



def upload_qiniu_name(instance, filename):
    # 自定义七牛上传文件名
    return 'vote/%(datepath)s/%(filename)s' % {'datepath':datetime.datetime.today().strftime('%Y/%m/%d'), 'filename':unicode(uuid.uuid1())}


####################################
# 图片 #
####################################
class VoteImage(models.Model):
    image = models.ImageField(storage=QiniuMediaStorage(), upload_to=upload_qiniu_name, max_length=256, null=True)

    def __unicode__(self):
        return unicode(self.image.url)



####################################
# 微信用户信息 #
####################################
class WeChatUser(models.Model):
    openid = models.CharField(default='', max_length=32, primary_key=True) # 用户的唯一标识
    nickname = models.CharField(default='', max_length=32) # 用户昵称
    # 用户的性别，值为1时是男性，值为2时是女性，值为0时是未知
    sex = models.CharField(default='0', max_length=1, choices=(('0', 'unknown'), ('1', 'man'),('2', 'female')))
    province = models.CharField(default='', max_length=16) #    用户个人资料填写的省份
    city = models.CharField(default='', max_length=16) #    普通用户个人资料填写的城市
    country = models.CharField(default='', max_length=16) # 国家，如中国为CN
    # 用户头像，最后一个数值代表正方形头像大小（有0、46、64、96、132数值可选，0代表640*640正方形头像），用户没有头像时该项为空。若用户更换头像，原有头像URL将失效。
    headimgurl = models.CharField(default='', max_length=256)


####################################
# 投票活动 #
####################################
class VoteActivity(AbstractDefaultColumn):
    # 活动名
    name = models.CharField(default='', max_length=32)
    # 背景图（不同的投票活动可能只有背景图和活动详情图不一样，其他流程都一样）
    background_image = models.ForeignKey('VoteImage', related_name='activity_bg_image', blank=True, null=True, on_delete=models.SET_NULL)
    # 活动详情图
    detailinfo_image = models.ForeignKey('VoteImage', related_name='activity_di_image', blank=True, null=True, on_delete=models.SET_NULL)
    # 更多图片
    images = models.ManyToManyField('VoteImage', related_name='activity_images', blank=True)

    # 每个用户对一个活动最多可以上传几个作品（0表示无限制）
    up_product_max = models.PositiveIntegerField(default=0)
    # 每个用户对一个活动最多可以投几次票（0表示无限制）
    vote_activity_max = models.PositiveIntegerField(default=0)
    # 每个用户对一个作品最多可以投几次票（0表示无限制）
    vote_product_max = models.PositiveIntegerField(default=0)
    # 是否可以给自己投票
    vote_self_enabled = models.BooleanField(default=False)
    # 活动截止时间
    end_time = models.DateTimeField(default=timezone.now)
    # 是否验证活动截止时间
    is_endtime_active = models.BooleanField(default=False)

    # 是否可用
    active = models.BooleanField(default=True)

    # 获取活动页面地址
    def get_url(self):
        pass

    def __unicode__(self):
        return unicode(self.name)


####################################
# 投票上传 #
####################################
class VoteProduct(AbstractDefaultColumn):
    # 活动id
    activity_id = models.ForeignKey('VoteActivity', related_name='product_activity', blank=True, null=True, on_delete=models.SET_NULL)
    # 活动名
    activity_name = models.CharField(default='', max_length=32)
    # 上传图片
    images = models.ManyToManyField('VoteImage', related_name='product_images')
    # 上传的标题
    title = models.CharField(default='', max_length=32)
    # 上传的文字内容或备注
    content = models.CharField(default='', max_length=128)
    # 票数
    vote_count = models.PositiveIntegerField(default=0)
    # 上传用户的相关信息
    up_user_slug = models.CharField(default='', max_length=32) # 作为标志唯一用户
    up_user_name = models.CharField(default='', max_length=32)
    up_user_mobile = models.CharField(default='', max_length=11)
    up_user_email = models.EmailField(default='')
    wechatuser = models.ForeignKey('WeChatUser', related_name='product_wechatuser', blank=True, null=True, on_delete=models.SET_NULL)

    # 是否可用
    active = models.BooleanField(default=True)

    @property
    def image_first(self):
        return self.images.first()

    @property
    def image_first_url(self):
        image = self.images.first()
        return (image.image.url if image else '')

    @property
    def slug(self):
        return '%04d' % self.id

    


    class Meta:
        ordering = ['-id'] # 默认排序

    def __unicode__(self):
        return unicode('%(activity_name)s %(product_title)s' % {'activity_name':self.activity_name, 'product_title':self.title})



####################################
# 投票记录 #
####################################
class VoteItem(AbstractDefaultColumn):
    # 投票对象id
    product_id = models.ForeignKey('VoteProduct', related_name='item_activity', blank=True, null=True, on_delete=models.SET_NULL)
    # 投票活动id（冗余，方便后面统计使用）
    activity_id = models.ForeignKey('VoteActivity', related_name='item_activity', blank=True, null=True, on_delete=models.SET_NULL)
    # 备注
    note = models.CharField(default='', max_length=32)
    # 投票用户相关信息
    vote_user_slug = models.CharField(default='', max_length=32) # 作为标志唯一用户
    vote_user_name = models.CharField(default='', max_length=32)
    vote_user_mobile = models.CharField(default='', max_length=11)
    vote_user_email = models.EmailField(default='')
    wechatuser = models.ForeignKey('WeChatUser', related_name='item_wechatuser', blank=True, null=True, on_delete=models.SET_NULL)
